@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
+require_relative "spec_helper"
 
 describe "Model#values" do
   before do
@@ -33,7 +33,7 @@ end
 
 describe "Model#save server use" do
   before do
-    @db = Sequel.mock(:autoid=>proc{|sql| 10}, :fetch=>{:x=>1, :id=>10}, :servers=>{:blah=>{}, :read_only=>{}})
+    @db = Sequel.mock(:autoid=>proc{|sql| 10}, :fetch=>{:x=>1, :id=>10}, :numrows=>1, :servers=>{:blah=>{}, :read_only=>{}})
     @c = Class.new(Sequel::Model(@db[:items]))
     @c.columns :id, :x, :y
     @c.dataset.columns(:id, :x, :y)
@@ -50,6 +50,19 @@ describe "Model#save server use" do
     @c.new(:x=>1).save.must_equal @c.load(:x=>1, :id=>10)
     @db.sqls.must_equal ["INSERT INTO items (x) VALUES (1) -- blah", 'SELECT * FROM items WHERE (id = 10) LIMIT 1 -- blah']
   end
+
+  it "should use transactions on the correct server"  do
+    @c.use_transactions = true
+    @c.dataset = @c.dataset.server(:blah)
+    @c.new(:x=>1).save.must_equal @c.load(:x=>1, :id=>10)
+    @db.sqls.must_equal ["BEGIN -- blah", "INSERT INTO items (x) VALUES (1) -- blah", 'SELECT * FROM items WHERE (id = 10) LIMIT 1 -- blah', 'COMMIT -- blah']
+
+    o = @c.load(:id=>1)
+    o.x = 2
+    o.this
+    o.save
+    @db.sqls.must_equal ["BEGIN -- blah", "UPDATE items SET x = 2 WHERE (id = 1) -- blah", 'COMMIT -- blah']
+  end
 end
 
 describe "Model#save" do
@@ -87,13 +100,41 @@ describe "Model#save" do
     DB.sqls.must_equal ["INSERT INTO items (y) VALUES (2) RETURNING *"]
   end
 
+  it "should issue regular insert query if insert_select returns nil" do
+    @c.dataset = @c.dataset.with_fetch(:id=>4, :x=>2).with_autoid(4).with_extend do
+      def supports_insert_select?; true end
+      def insert_select(hash)
+      end
+    end
+    o = @c.new(:x => 1)
+    o.save
+    
+    DB.sqls.must_equal ["INSERT INTO items (x) VALUES (1)", "SELECT * FROM items WHERE id = 4"]
+    o.values.must_equal(:id=>4, :x=>2)
+  end
+
+  it "should assume insert statement already ran if insert_select returns false" do
+    @c.dataset = @c.dataset.with_fetch(:y=>2).with_extend do
+      def supports_insert_select?; true end
+      def insert_select(hash)
+        with_sql_first("INSERT INTO items (y) VALUES (2) RETURNING *")
+        false
+      end
+    end
+    o = @c.new(:x => 1)
+    o.save
+    
+    o.values.must_equal(:x=>1)
+    DB.sqls.must_equal ["INSERT INTO items (y) VALUES (2) RETURNING *"]
+  end
+
   it "should not use dataset's insert_select method if specific columns are selected" do
-    ds = @c.dataset = @c.dataset.select(:y).with_extend{def insert_select(*) raise; end}
+    @c.dataset = @c.dataset.select(:y).with_extend{def insert_select(*) raise; end}
     @c.new(:x => 1).save
   end
 
   it "should use dataset's insert_select method if the dataset uses returning, even if specific columns are selected" do
-    ds = @c.dataset = @c.dataset.select(:y).with_fetch(:y=>2).with_extend do
+    @c.dataset = @c.dataset.select(:y).with_fetch(:y=>2).with_extend do
       def supports_returning?(_) true end
       def supports_insert_select?; true end
       def insert_select(hash)
@@ -133,10 +174,8 @@ describe "Model#save" do
     o = @c.new(:x => 11)
     def o.autoincrementing_primary_key() :y end
     o.save
-    sqls = DB.sqls
-    sqls.length.must_equal 2
-    sqls.first.must_equal "INSERT INTO items (x) VALUES (11)"
-    sqls.last.must_match %r{SELECT \* FROM items WHERE \(\([xy] = 1[13]\) AND \([xy] = 1[13]\)\) LIMIT 1}
+    DB.sqls.must_equal ["INSERT INTO items (x) VALUES (11)",
+      'SELECT * FROM items WHERE ((x = 11) AND (y = 13)) LIMIT 1']
   end
 
   it "should update a record for an existing model instance" do
@@ -187,7 +226,7 @@ describe "Model#save" do
   end
   
   it "should mark all columns as not changed if this is a new record and insert_select was used" do
-    ds = @c.dataset = @c.dataset.with_extend{def insert_select(h) h.merge(:id=>1) end}
+    @c.dataset = @c.dataset.with_extend{def insert_select(h) h.merge(:id=>1) end}
     o = @c.new(:x => 1, :y => nil)
     o.x = 4
     o.changed_columns.must_equal [:x]
@@ -195,34 +234,6 @@ describe "Model#save" do
     o.changed_columns.must_equal []
   end
 
-  it "should store previous value of @new in @was_new and as well as the hash used for updating in @columns_updated until after hooks finish running" do
-    res = nil
-    @c.send(:define_method, :after_save){ res = [@columns_updated, @was_new]}
-    o = @c.new(:x => 1, :y => nil)
-    o[:x] = 2
-    o.save
-    res.must_equal [nil, true]
-    o.after_save
-    res.must_equal [nil, nil]
-
-    res = nil
-    o = @c.load(:id => 23,:x => 1, :y => nil)
-    o[:x] = 2
-    o.save
-    res.must_equal [{:x => 2, :y => nil}, nil]
-    o.after_save
-    res.must_equal [nil, nil]
-
-    res = nil
-    o = @c.load(:id => 23,:x => 2, :y => nil)
-    o[:x] = 2
-    o[:y] = 22
-    o.save(:columns=>:x)
-    res.must_equal [{:x=>2},nil]
-    o.after_save
-    res.must_equal [nil, nil]
-  end
-  
   it "should use Model's use_transactions setting by default" do
     @c.use_transactions = true
     @c.load(:id => 3, :x => 1, :y => nil).save(:columns=>:y)
@@ -265,17 +276,6 @@ describe "Model#save" do
     DB.sqls.must_equal ["BEGIN", "UPDATE items SET y = NULL WHERE (id = 3)", "COMMIT"]
   end
 
-  it "should rollback if before_save returns false and raise_on_save_failure = true" do
-    o = @c.load(:id => 3, :x => 1, :y => nil)
-    o.use_transactions = true
-    o.raise_on_save_failure = true
-    def o.before_save
-      false
-    end
-    proc { o.save(:columns=>:y) }.must_raise(Sequel::HookFailed)
-    DB.sqls.must_equal ["BEGIN", "ROLLBACK"]
-  end
-
   it "should rollback if before_save calls cancel_action and raise_on_save_failure = true" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.use_transactions = true
@@ -287,23 +287,23 @@ describe "Model#save" do
     DB.sqls.must_equal ["BEGIN", "ROLLBACK"]
   end
 
-  it "should rollback if before_save returns false and :raise_on_failure option is true" do
+  it "should rollback if before_save calls cancel_action and :raise_on_failure option is true" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.use_transactions = true
     o.raise_on_save_failure = false
     def o.before_save
-      false
+      cancel_action
     end
     proc { o.save(:columns=>:y, :raise_on_failure => true) }.must_raise(Sequel::HookFailed)
     DB.sqls.must_equal ["BEGIN", "ROLLBACK"]
   end
 
-  it "should not rollback outer transactions if before_save returns false and raise_on_save_failure = false" do
+  it "should not rollback outer transactions if before_save calls cancel_action and raise_on_save_failure = false" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.use_transactions = true
     o.raise_on_save_failure = false
     def o.before_save
-      false
+      cancel_action
     end
     DB.transaction do
       o.save(:columns=>:y).must_be_nil
@@ -312,12 +312,12 @@ describe "Model#save" do
     DB.sqls.must_equal ["BEGIN", "BLAH", "COMMIT"]
   end
 
-  it "should rollback if before_save returns false and raise_on_save_failure = false" do
+  it "should rollback if before_save calls cancel_action and raise_on_save_failure = false" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.use_transactions = true
     o.raise_on_save_failure = false
     def o.before_save
-      false
+      cancel_action
     end
     o.save(:columns=>:y).must_be_nil
     DB.sqls.must_equal ["BEGIN", "ROLLBACK"]
@@ -433,7 +433,7 @@ describe "Model#freeze" do
   end
 
   it "should still have working class attr overriddable methods" do
-    Sequel::Model::BOOLEAN_SETTINGS.each{|m| @o.send(m) == Album.send(m)}
+    [:typecast_empty_string_to_nil, :typecast_on_assignment, :strict_param_setting, :raise_on_save_failure, :raise_on_typecast_failure, :require_modification, :use_transactions].each{|m| @o.send(m) == Album.send(m)}
   end
 
   it "should have working new? method" do
@@ -683,12 +683,14 @@ describe "Model#save_changes" do
   end
 
   it "should take options passed to save" do
-    o = @c.new(:x => 1)
-    def o.before_validation; false; end
-    proc{o.save_changes}.must_raise(Sequel::HookFailed)
-    DB.sqls.must_equal []
-    o.save_changes(:validate=>false)
-    DB.sqls.first.must_equal "INSERT INTO items (x) VALUES (1)"
+    o = @c.load(:id=>1, :x => 1)
+    o.x = 2
+    o.save_changes
+    DB.sqls.must_equal ["UPDATE items SET x = 2 WHERE (id = 1)"]
+
+    o.x = 3
+    o.save_changes(:transaction=>true)
+    DB.sqls.must_equal ["BEGIN", "UPDATE items SET x = 3 WHERE (id = 1)", "COMMIT"]
   end
 
   it "should do nothing if no changed columns" do
@@ -846,16 +848,22 @@ describe Sequel::Model, "#this" do
     instance.this.sql.must_equal "SELECT * FROM examples WHERE (a = 3) LIMIT 1"
   end
 
-  it "should use a qualified primary key if the dataset is joined" do
+  it "should use a subquery if the dataset is joined" do
     @example.dataset = @example.dataset.cross_join(:a)
     instance = @example.load(:id => 3)
-    instance.this.sql.must_equal "SELECT * FROM examples CROSS JOIN a WHERE (examples.id = 3) LIMIT 1"
+    instance.this.sql.must_equal "SELECT * FROM (SELECT * FROM examples CROSS JOIN a) AS examples WHERE (id = 3) LIMIT 1"
+  end
+
+  it "should use a primary key if the dataset uses a subquery" do
+    @example.dataset = @example.dataset.cross_join(:a).from_self(:alias=>:b)
+    instance = @example.load(:id => 3)
+    instance.this.sql.must_equal "SELECT * FROM (SELECT * FROM examples CROSS JOIN a) AS b WHERE (id = 3) LIMIT 1"
   end
 
   it "should support composite primary keys" do
     @example.set_primary_key [:x, :y]
     instance = @example.load(:x => 4, :y => 5)
-    instance.this.sql.must_match(/SELECT \* FROM examples WHERE \(\([xy] = [45]\) AND \([xy] = [45]\)\) LIMIT 1/)
+    instance.this.sql.must_equal 'SELECT * FROM examples WHERE ((x = 4) AND (y = 5)) LIMIT 1'
   end
 end
 
@@ -1035,7 +1043,7 @@ describe Sequel::Model, "#set" do
 
   it "should raise error if strict_param_setting is true and column is restricted" do
     @o1.strict_param_setting = true
-    @c.set_allowed_columns
+    @c.setter_methods.delete("x=")
     proc{@o1.set('x' => 1)}.must_raise(Sequel::MassAssignmentRestriction)
   end
 
@@ -1241,9 +1249,7 @@ describe Sequel::Model, "#update_fields" do
   it "should set only the given fields, and then save the changes to the record" do
     @o1.update_fields({:x => 1, :y => 2, :z=>3, :id=>4}, [:x, :y])
     @o1.values.must_equal(:x => 1, :y => 2, :id=>1)
-    sqls = DB.sqls
-    sqls.pop.must_match(/UPDATE items SET [xy] = [12], [xy] = [12] WHERE \(id = 1\)/)
-    sqls.must_equal []
+    DB.sqls.must_equal ['UPDATE items SET x = 1, y = 2 WHERE (id = 1)']
 
     @o1.update_fields({:x => 1, :y => 5, :z=>6, :id=>7}, [:x, :y])
     @o1.values.must_equal(:x => 1, :y => 5, :id=>1)
@@ -1272,69 +1278,6 @@ describe Sequel::Model, "#update_fields" do
   end
 end
 
-describe Sequel::Model, "#(set|update)_(all|only)" do
-  before do
-    @c = Class.new(Sequel::Model(:items)) do
-      set_primary_key :id
-      columns :x, :y, :z, :id
-      set_allowed_columns :x
-    end
-    @c.strict_param_setting = false
-    @o1 = @c.new
-    DB.reset
-  end
-
-  it "should raise errors if not all hash fields can be set and strict_param_setting is true" do
-    @c.strict_param_setting = true
-
-    proc{@c.new.set_all(:x => 1, :y => 2, :z=>3, :use_after_commit_rollback => false)}.must_raise(Sequel::MassAssignmentRestriction)
-    (o = @c.new).set_all(:x => 1, :y => 2, :z=>3)
-    o.values.must_equal(:x => 1, :y => 2, :z=>3)
-
-    proc{@c.new.set_only({:x => 1, :y => 2, :z=>3, :id=>4}, :x, :y)}.must_raise(Sequel::MassAssignmentRestriction)
-    proc{@c.new.set_only({:x => 1, :y => 2, :z=>3}, :x, :y)}.must_raise(Sequel::MassAssignmentRestriction)
-    (o = @c.new).set_only({:x => 1, :y => 2}, :x, :y)
-    o.values.must_equal(:x => 1, :y => 2)
-  end
-
-  it "#set_all should set all attributes including the primary key" do
-    @o1.set_all(:x => 1, :y => 2, :z=>3, :id=>4)
-    @o1.values.must_equal(:id =>4, :x => 1, :y => 2, :z=>3)
-  end
-
-  it "#set_all should set not set restricted fields" do
-    @o1.use_after_commit_rollback.must_be_nil
-    @o1.set_all(:x => 1, :use_after_commit_rollback => true)
-    @o1.use_after_commit_rollback.must_be_nil
-    @o1.values.must_equal(:x => 1)
-  end
-
-  it "#set_only should only set given attributes" do
-    @o1.set_only({:x => 1, :y => 2, :z=>3, :id=>4}, [:x, :y])
-    @o1.values.must_equal(:x => 1, :y => 2)
-    @o1.set_only({:x => 4, :y => 5, :z=>6, :id=>7}, :x, :y)
-    @o1.values.must_equal(:x => 4, :y => 5)
-    @o1.set_only({:x => 9, :y => 8, :z=>6, :id=>7}, :x, :y, :id)
-    @o1.values.must_equal(:x => 9, :y => 8, :id=>7)
-  end
-
-  it "#update_all should update all attributes" do
-    @c.new.update_all(:x => 1)
-    DB.sqls.must_equal ["INSERT INTO items (x) VALUES (1)", "SELECT * FROM items WHERE id = 10"]
-    @c.new.update_all(:y => 1)
-    DB.sqls.must_equal ["INSERT INTO items (y) VALUES (1)", "SELECT * FROM items WHERE id = 10"]
-    @c.new.update_all(:z => 1)
-    DB.sqls.must_equal ["INSERT INTO items (z) VALUES (1)", "SELECT * FROM items WHERE id = 10"]
-  end
-
-  it "#update_only should only update given attributes" do
-    @o1.update_only({:x => 1, :y => 2, :z=>3, :id=>4}, [:x])
-    DB.sqls.must_equal ["INSERT INTO items (x) VALUES (1)", "SELECT * FROM items WHERE id = 10"]
-    @c.new.update_only({:x => 1, :y => 2, :z=>3, :id=>4}, :x)
-    DB.sqls.must_equal ["INSERT INTO items (x) VALUES (1)", "SELECT * FROM items WHERE id = 10"]
-  end
-end
-
 describe Sequel::Model, "#destroy with filtered dataset" do
   before do
     @model = Class.new(Sequel::Model(DB[:items].where(:a=>1)))
